import ij.ImagePlus;
import ij.gui.GenericDialog;
import ij.plugin.PlugIn;
import imagescience.image.Image;
import imagescience.transform.Shift;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Panel;

public class TJ_Shift implements PlugIn {

    private static String xshift = "0.0";
    private static String yshift = "0.0";
    private static String zshift = "0.0";

    private static final String[] schemes = {
	"nearest neighbor",
	"linear",
	"cubic convolution",
	"cubic B-spline",
	"cubic O-MOMS",
	"quintic B-spline"
    };
    private static int scheme = 1;

    private static String bgvalue = "0.0";

    public void run(String arg) {

	if (!TJ.libcheck()) return;
	final ImagePlus imp = TJ.imageplus();
	if (imp == null) return;

	TJ.debug(TJ.name()+" "+TJ.version()+": Shift");

	GenericDialog gd = new GenericDialog(TJ.name()+": Shift");
	gd.addStringField("x-shift in pixels:",xshift);
	gd.addStringField("y-shift in pixels:",yshift);
	gd.addStringField("z-shift in slices:",zshift);
	gd.addPanel(new Panel(),GridBagConstraints.WEST,new Insets(0,0,0,0));
	gd.addChoice("Interpolation scheme:",schemes,schemes[scheme]);
	gd.addStringField("Background value:",bgvalue);
	gd.showDialog();

	if (gd.wasCanceled()) return;

	xshift = gd.getNextString();
	yshift = gd.getNextString();
	zshift = gd.getNextString();
	scheme = gd.getNextChoiceIndex();
	bgvalue = gd.getNextString();

	(new TJShift()).run(imp,xshift,yshift,zshift,scheme,bgvalue);
    }

}

class TJShift { // Separate class to make the plugin class independent of imagescience.jar

    void run(final ImagePlus imp,
	     final String xshift,
	     final String yshift,
	     final String zshift,
	     final int scheme,
	     final String bgvalue) {

	try {
	    final Image img = Image.wrap(imp);
	    final Shift shifter = new Shift();
	    shifter.debug(TJ_Options.debug);
	    double xs, ys, zs, bg;
	    try { xs = Double.parseDouble(xshift); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid x-shift value"); }
	    try { ys = Double.parseDouble(yshift); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid y-shift value"); }
	    try { zs = Double.parseDouble(zshift); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid z-shift value"); }
	    try { bg = Double.parseDouble(bgvalue); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid background value"); }
	    shifter.background(bg);
	    int ischeme = Shift.NEAREST;
	    switch (scheme) {
	    case 0: ischeme = Shift.NEAREST; break;
	    case 1: ischeme = Shift.LINEAR; break;
	    case 2: ischeme = Shift.CUBIC; break;
	    case 3: ischeme = Shift.BSPLINE3; break;
	    case 4: ischeme = Shift.OMOMS3; break;
	    case 5: ischeme = Shift.BSPLINE5; break;
	    }
	    final Image newimg = shifter.run(img,xs,ys,zs,ischeme);
	    TJ.show(newimg,imp);

	} catch (OutOfMemoryError e) {
	    TJ.error("Not enough memory for this operation");

	} catch (UnknownError e) {
	    TJ.error("Could not create output image for some reason.\nPossibly there is not enough free memory");

	} catch (IllegalArgumentException e) {
	    TJ.error(e.getMessage());

	} catch (Throwable e) {
	    TJ.error("An unidentified error occurred while running the plugin");

	}
    }

}
